@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,7 +35,6 @@
 #include <errno.h>
 
 #ifndef G_OS_WIN32
-static const char *path = "/usr/bin/gpg";
 static const char *passphrase = "no.secret";
 
 static gboolean
@@ -68,7 +67,7 @@ parse_message (int fd)
 	g_object_unref (stream);
 	
 	/* parse the message from the stream */
-	message = g_mime_parser_construct_message (parser);
+	message = g_mime_parser_construct_message (parser, NULL);
 	
 	/* free the parser (and the stream) */
 	g_object_unref (parser);
@@ -136,37 +135,19 @@ count_parts_in_message (GMimeMessage *message)
 }
 
 #ifndef G_OS_WIN32
-static GMimeSignatureStatus
-sig_status (GMimeSignatureList *signatures)
-{
-	GMimeSignatureStatus status = GMIME_SIGNATURE_STATUS_GOOD;
-	GMimeSignature *sig;
-	int i;
-	
-	if (!signatures || signatures->array->len == 0)
-		return GMIME_SIGNATURE_STATUS_ERROR;
-	
-	for (i = 0; i < g_mime_signature_list_length (signatures); i++) {
-		sig = g_mime_signature_list_get_signature (signatures, i);
-		status = MAX (status, sig->status);
-	}
-	
-	return status;
-}
-
 static void
 verify_foreach_callback (GMimeObject *parent, GMimeObject *part, gpointer user_data)
 {
-	GMimeCryptoContext *ctx = user_data;
-	
 	if (GMIME_IS_MULTIPART_SIGNED (part)) {
 		/* this is a multipart/signed part, so we can verify the pgp signature */
 		GMimeMultipartSigned *mps = (GMimeMultipartSigned *) part;
 		GMimeSignatureList *signatures;
+		GMimeSignature *sig;
 		GError *err = NULL;
 		const char *str;
+		int i;
 		
-		if (!(signatures = g_mime_multipart_signed_verify (mps, ctx, &err))) {
+		if (!(signatures = g_mime_multipart_signed_verify (mps, GMIME_VERIFY_NONE, &err))) {
 			/* an error occured - probably couldn't start gpg? */
 			
 			/* for more information about GError, see:
@@ -177,19 +158,15 @@ verify_foreach_callback (GMimeObject *parent, GMimeObject *part, gpointer user_d
 			g_error_free (err);
 		} else {
 			/* print out validity info - GOOD vs BAD and "why" */
-			switch (sig_status (signatures)) {
-			case GMIME_SIGNATURE_STATUS_GOOD:
-				str = "Good";
-				break;
-			case GMIME_SIGNATURE_STATUS_BAD:
-				str = "Bad";
-				break;
-			case GMIME_SIGNATURE_STATUS_ERROR:
-				str = "Error";
-				break;
-			default:
-				str = NULL;
-				break;
+			for (i = 0; i < g_mime_signature_list_length (signatures); i++) {
+				sig = g_mime_signature_list_get_signature (signatures, i);
+				
+				if ((sig->status & GMIME_SIGNATURE_STATUS_RED) != 0)
+					str = "Bad";
+				else if ((sig->status & GMIME_SIGNATURE_STATUS_GREEN) != 0)
+					str = "Good";
+				else
+					str = "Error";
 			}
 			
 			g_object_unref (signatures);
@@ -198,10 +175,10 @@ verify_foreach_callback (GMimeObject *parent, GMimeObject *part, gpointer user_d
 }
 
 static void
-verify_signed_parts (GMimeMessage *message, GMimeCryptoContext *ctx)
+verify_signed_parts (GMimeMessage *message)
 {
 	/* descend the mime tree and verify any signed parts */
-	g_mime_message_foreach (message, verify_foreach_callback, ctx);
+	g_mime_message_foreach (message, verify_foreach_callback, NULL);
 }
 #endif
 
@@ -215,7 +192,7 @@ write_message_to_screen (GMimeMessage *message)
 	g_mime_stream_file_set_owner ((GMimeStreamFile *) stream, FALSE);
 	
 	/* write the message to the stream */
-	g_mime_object_write_to_stream ((GMimeObject *) message, stream);
+	g_mime_object_write_to_stream ((GMimeObject *) message, NULL, stream);
 	
 	/* flush the stream (kinda like fflush() in libc's stdio) */
 	g_mime_stream_flush (stream);
@@ -224,35 +201,24 @@ write_message_to_screen (GMimeMessage *message)
 	g_object_unref (stream);
 }
 
-#define PART_CONTENT "Hello, this is the new text/plain part's content text."
+#define TEXT_CONTENT "Hello, this is the new text/plain part's content text."
 
 static void
 add_a_mime_part (GMimeMessage *message)
 {
 	GMimeMultipart *multipart;
 	GMimeDataWrapper *content;
-	GMimePart *mime_part;
+	GMimeTextPart *mime_part;
 	GMimeStream *stream;
 	
 	/* create the new part that we are going to add... */
-	mime_part = g_mime_part_new_with_type ("text", "plain");
+	mime_part = g_mime_text_part_new_with_subtype ("plain");
 	
-	/* create the parts' content stream */
-	stream = g_mime_stream_mem_new_with_buffer (PART_CONTENT, strlen (PART_CONTENT));
-	
-	/* create the content object - since the stream is not base64
-	   or QP encoded or anything, we can use
-	   GMIME_CONTENT_ENCODING_DEFAULT as the encoding type (_DEFAULT
-	   is the same as saying "nothing specified") */
-	content = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_DEFAULT);
-	g_object_unref (stream);
-	
-	/* set the content object on the new mime part */
-	g_mime_part_set_content_object (mime_part, content);
-	g_object_unref (content);
+	/* set the text content of the mime part */
+	g_mime_text_part_set_text (mime_part, TEXT_CONTENT);
 	
 	/* if we want, we can tell GMime that the content should be base64 encoded when written to disk... */
-	g_mime_part_set_content_encoding (mime_part, GMIME_CONTENT_ENCODING_BASE64);
+	g_mime_part_set_content_encoding ((GMimePart *) mime_part, GMIME_CONTENT_ENCODING_BASE64);
 	
 	/* the "polite" way to modify a mime structure that we didn't
 	   create is to create a new toplevel multipart/mixed part and
@@ -304,36 +270,30 @@ int main (int argc, char **argv)
 	if (argc < 2) {
 		printf ("Usage: a.out <message file>\n");
 		return 0;
-	} else {
-		if ((fd = open (argv[1], O_RDONLY, 0)) == -1) {
-			fprintf (stderr, "Cannot open message `%s': %s\n", argv[1], g_strerror (errno));
-			return 0;
-		}
+	}
+	
+	if ((fd = open (argv[1], O_RDONLY, 0)) == -1) {
+		fprintf (stderr, "Cannot open message `%s': %s\n", argv[1], g_strerror (errno));
+		return 0;
 	}
 	
 	/* init the gmime library */
-	g_mime_init (0);
+	g_mime_init ();
 	
 	/* parse the message */
 	message = parse_message (fd);
+	if (message == NULL) {
+		printf ("Error parsing message\n");
+		return 1;
+	}
 	
 	/* count the number of parts in the message */
 	count_parts_in_message (message);
 	
 #ifndef G_OS_WIN32
 #ifdef ENABLE_CRYPTOGRAPHY
-	/* create our crypto context */
-	ctx = g_mime_gpg_context_new (request_passwd, path);
-	
-	/* don't allow auto key-retrival */
-	g_mime_gpg_context_set_auto_key_retrieve ((GMimeGpgContext *) ctx, FALSE);
-	
-	/* set the always_trust flag so that gpg will be spawned with `gpg --always-trust` */
-	g_mime_gpg_context_set_always_trust ((GMimeGpgContext *) ctx, TRUE);
-	
 	/* verify any signed parts */
-	verify_signed_parts (message, ctx);
-	g_object_unref (ctx);
+	verify_signed_parts (message);
 #endif
 #endif
 	
