@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -27,11 +27,12 @@
 #include <string.h>
 
 #include "gmime-multipart-encrypted.h"
+#include "gmime-filter-dos2unix.h"
 #include "gmime-stream-filter.h"
 #include "gmime-filter-basic.h"
 #include "gmime-filter-from.h"
-#include "gmime-filter-crlf.h"
 #include "gmime-stream-mem.h"
+#include "gmime-internal.h"
 #include "gmime-parser.h"
 #include "gmime-part.h"
 #include "gmime-error.h"
@@ -56,7 +57,6 @@
  * multipart/encrypted MIME type.
  **/
 
-
 /* GObject class methods */
 static void g_mime_multipart_encrypted_class_init (GMimeMultipartEncryptedClass *klass);
 static void g_mime_multipart_encrypted_init (GMimeMultipartEncrypted *mps, GMimeMultipartEncryptedClass *klass);
@@ -127,10 +127,10 @@ g_mime_multipart_encrypted_new (void)
 	GMimeMultipartEncrypted *multipart;
 	GMimeContentType *content_type;
 	
-	multipart = g_object_newv (GMIME_TYPE_MULTIPART_ENCRYPTED, 0, NULL);
+	multipart = g_object_new (GMIME_TYPE_MULTIPART_ENCRYPTED, NULL);
 	
 	content_type = g_mime_content_type_new ("multipart", "encrypted");
-	g_mime_object_set_content_type (GMIME_OBJECT (multipart), content_type);
+	g_mime_object_set_content_type ((GMimeObject *) multipart, content_type);
 	g_object_unref (content_type);
 	
 	return multipart;
@@ -139,252 +139,232 @@ g_mime_multipart_encrypted_new (void)
 
 /**
  * g_mime_multipart_encrypted_encrypt:
- * @mpe: multipart/encrypted object
- * @content: MIME part to encrypt
- * @ctx: encryption context
+ * @ctx: a #GMimeCryptoContext
+ * @entity: MIME part to encrypt
  * @sign: %TRUE if the content should also be signed or %FALSE otherwise
- * @userid: user id to use for signing (only used if @sign is %TRUE)
- * @digest: digest algorithm to use when signing
+ * @userid: (nullable): user id to use for signing (only used if @sign is %TRUE)
+ * @flags: a #GMimeEncryptFlags
  * @recipients: (element-type utf8): an array of recipients to encrypt to
  * @err: a #GError
  *
- * Attempts to encrypt (and conditionally sign) the @content MIME part
+ * Attempts to encrypt (and conditionally sign) the @entity MIME part
  * to the public keys of @recipients using the @ctx encryption
- * context. If successful, the encrypted #GMimeObject is set as the
- * encrypted part of the multipart/encrypted object @mpe.
+ * context. If successful, a new multipart/encrypted object is returned.
  *
- * Returns: %0 on success or %-1 on fail. If the encryption fails, an
- * exception will be set on @err to provide information as to why the
- * failure occured.
+ * Returns: (nullable) (transfer full): a new #GMimeMultipartEncrypted object on success
+ * or %NULL on fail. If encrypting fails, an exception will be set on @err to provide
+ * information as to why the failure occured.
  **/
-int
-g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *content,
-				    GMimeCryptoContext *ctx, gboolean sign,
-				    const char *userid, GMimeDigestAlgo digest,
-				    GPtrArray *recipients, GError **err)
+GMimeMultipartEncrypted *
+g_mime_multipart_encrypted_encrypt (GMimeCryptoContext *ctx, GMimeObject *entity, gboolean sign, const char *userid,
+				    GMimeEncryptFlags flags, GPtrArray *recipients, GError **err)
 {
-	GMimeStream *filtered_stream, *ciphertext, *stream;
 	GMimePart *version_part, *encrypted_part;
+	GMimeMultipartEncrypted *encrypted;
+	GMimeStream *stream, *ciphertext;
 	GMimeContentType *content_type;
-	GMimeDataWrapper *wrapper;
-	GMimeFilter *crlf_filter;
+	GMimeFormatOptions *options;
+	GMimeDataWrapper *content;
 	const char *protocol;
+	GMimeFilter *filter;
 	
-	g_return_val_if_fail (GMIME_IS_MULTIPART_ENCRYPTED (mpe), -1);
-	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), -1);
-	g_return_val_if_fail (GMIME_IS_OBJECT (content), -1);
+	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
+	g_return_val_if_fail (GMIME_IS_OBJECT (entity), NULL);
 	
 	if (!(protocol = g_mime_crypto_context_get_encryption_protocol (ctx))) {
 		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED, _("Encryption not supported."));
-		return -1;
+		return NULL;
 	}
 	
 	/* get the cleartext */
 	stream = g_mime_stream_mem_new ();
-	filtered_stream = g_mime_stream_filter_new (stream);
 	
-	crlf_filter = g_mime_filter_crlf_new (TRUE, FALSE);
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), crlf_filter);
-	g_object_unref (crlf_filter);
+	options = _g_mime_format_options_clone (NULL, FALSE);
+	g_mime_format_options_set_newline_format (options, GMIME_NEWLINE_FORMAT_DOS);
 	
-	g_mime_object_write_to_stream (content, filtered_stream);
-	g_mime_stream_flush (filtered_stream);
-	g_object_unref (filtered_stream);
+	g_mime_object_write_to_stream (entity, options, stream);
 	
 	/* reset the content stream */
 	g_mime_stream_reset (stream);
 	
 	/* encrypt the content stream */
 	ciphertext = g_mime_stream_mem_new ();
-	if (g_mime_crypto_context_encrypt (ctx, sign, userid, digest, recipients, stream, ciphertext, err) == -1) {
+	if (g_mime_crypto_context_encrypt (ctx, sign, userid, flags, recipients, stream, ciphertext, err) == -1) {
 		g_object_unref (ciphertext);
 		g_object_unref (stream);
-		return -1;
+		return NULL;
 	}
 	
 	g_object_unref (stream);
 	g_mime_stream_reset (ciphertext);
 	
 	/* construct the version part */
-	content_type = g_mime_content_type_new_from_string (protocol);
+	content_type = g_mime_content_type_parse (NULL, protocol);
 	version_part = g_mime_part_new_with_type (content_type->type, content_type->subtype);
 	g_object_unref (content_type);
 	
-	content_type = g_mime_content_type_new_from_string (protocol);
-	g_mime_object_set_content_type (GMIME_OBJECT (version_part), content_type);
 	g_mime_part_set_content_encoding (version_part, GMIME_CONTENT_ENCODING_7BIT);
 	stream = g_mime_stream_mem_new_with_buffer ("Version: 1\n", strlen ("Version: 1\n"));
-	wrapper = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_7BIT);
-	g_mime_part_set_content_object (version_part, wrapper);
-	g_object_unref (wrapper);
+	content = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_7BIT);
+	g_mime_part_set_content (version_part, content);
+	g_object_unref (content);
 	g_object_unref (stream);
 	
 	/* construct the encrypted mime part */
 	encrypted_part = g_mime_part_new_with_type ("application", "octet-stream");
 	g_mime_part_set_content_encoding (encrypted_part, GMIME_CONTENT_ENCODING_7BIT);
-	wrapper = g_mime_data_wrapper_new_with_stream (ciphertext, GMIME_CONTENT_ENCODING_7BIT);
-	g_mime_part_set_content_object (encrypted_part, wrapper);
+	content = g_mime_data_wrapper_new_with_stream (ciphertext, GMIME_CONTENT_ENCODING_7BIT);
+	g_mime_part_set_content (encrypted_part, content);
 	g_object_unref (ciphertext);
-	g_object_unref (wrapper);
+	g_object_unref (content);
 	
 	/* save the version and encrypted parts */
-	/* FIXME: make sure there aren't any other parts?? */
-	g_mime_multipart_add (GMIME_MULTIPART (mpe), GMIME_OBJECT (version_part));
-	g_mime_multipart_add (GMIME_MULTIPART (mpe), GMIME_OBJECT (encrypted_part));
+	encrypted = g_mime_multipart_encrypted_new ();
+	g_mime_multipart_add ((GMimeMultipart *) encrypted, (GMimeObject *) version_part);
+	g_mime_multipart_add ((GMimeMultipart *) encrypted, (GMimeObject *) encrypted_part);
 	g_object_unref (encrypted_part);
 	g_object_unref (version_part);
 	
 	/* set the content-type params for this multipart/encrypted part */
-	g_mime_object_set_content_type_parameter (GMIME_OBJECT (mpe), "protocol", protocol);
-	g_mime_multipart_set_boundary (GMIME_MULTIPART (mpe), NULL);
+	g_mime_object_set_content_type_parameter ((GMimeObject *) encrypted, "protocol", protocol);
+	g_mime_multipart_set_boundary ((GMimeMultipart *) encrypted, NULL);
 	
-	return 0;
-}
-
-
-static GMimeStream *
-g_mime_data_wrapper_get_decoded_stream (GMimeDataWrapper *wrapper)
-{
-	GMimeStream *decoded_stream;
-	GMimeFilter *decoder;
-	
-	g_mime_stream_reset (wrapper->stream);
-	
-	switch (wrapper->encoding) {
-	case GMIME_CONTENT_ENCODING_BASE64:
-	case GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE:
-	case GMIME_CONTENT_ENCODING_UUENCODE:
-		decoder = g_mime_filter_basic_new (wrapper->encoding, FALSE);
-		decoded_stream = g_mime_stream_filter_new (wrapper->stream);
-		g_mime_stream_filter_add (GMIME_STREAM_FILTER (decoded_stream), decoder);
-		g_object_unref (decoder);
-		break;
-	default:
-		decoded_stream = wrapper->stream;
-		g_object_ref (wrapper->stream);
-		break;
-	}
-	
-	return decoded_stream;
+	return encrypted;
 }
 
 
 /**
  * g_mime_multipart_encrypted_decrypt:
- * @mpe: multipart/encrypted object
- * @ctx: decryption context
- * @result: a #GMimeDecryptionResult
+ * @encrypted: a #GMimeMultipartEncrypted
+ * @flags: a #GMimeDecryptFlags
+ * @session_key: session key to use or %NULL
+ * @result: (out) (transfer full): a #GMimeDecryptResult
  * @err: a #GError
  *
  * Attempts to decrypt the encrypted MIME part contained within the
- * multipart/encrypted object @mpe using the @ctx decryption context.
+ * multipart/encrypted object @encrypted.
+ *
+ * When non-%NULL, @session_key should be a %NULL-terminated string,
+ * such as the one returned by g_mime_decrypt_result_get_session_key()
+ * from a previous decryption. If the @sesion_key is not valid, decryption
+ * will fail.
  *
  * If @result is non-%NULL, then on a successful decrypt operation, it will be
  * updated to point to a newly-allocated #GMimeDecryptResult with signature
  * status information as well as a list of recipients that the part was
  * encrypted to.
  *
- * Returns: (transfer full): the decrypted MIME part on success or
+ * Returns: (nullable) (transfer full): the decrypted MIME part on success or
  * %NULL on fail. If the decryption fails, an exception will be set on
  * @err to provide information as to why the failure occured.
  **/
 GMimeObject *
-g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCryptoContext *ctx,
-				    GMimeDecryptResult **result, GError **err)
+g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *encrypted, GMimeDecryptFlags flags,
+				    const char *session_key, GMimeDecryptResult **result,
+				    GError **err)
 {
-	GMimeObject *decrypted, *version, *encrypted;
-	GMimeStream *stream, *ciphertext;
+	GMimeObject *decrypted, *version_part, *encrypted_part;
+	GMimeStream *filtered, *stream, *ciphertext;
 	const char *protocol, *supported;
-	GMimeStream *filtered_stream;
-	GMimeContentType *mime_type;
-	GMimeDataWrapper *wrapper;
-	GMimeFilter *crlf_filter;
+	GMimeContentType *content_type;
+	GMimeDataWrapper *content;
 	GMimeDecryptResult *res;
+	GMimeCryptoContext *ctx;
+	GMimeFilter *filter;
 	GMimeParser *parser;
-	char *content_type;
+	char *mime_type;
 	
-	g_return_val_if_fail (GMIME_IS_MULTIPART_ENCRYPTED (mpe), NULL);
-	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
+	g_return_val_if_fail (GMIME_IS_MULTIPART_ENCRYPTED (encrypted), NULL);
 	
 	if (result)
 		*result = NULL;
 	
-	protocol = g_mime_object_get_content_type_parameter (GMIME_OBJECT (mpe), "protocol");
-	supported = g_mime_crypto_context_get_encryption_protocol (ctx);
-	
-	if (protocol) {
-		/* make sure the protocol matches the crypto encrypt protocol */
-		if (!supported || g_ascii_strcasecmp (supported, protocol) != 0) {
-			g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
-				     _("Cannot decrypt multipart/encrypted part: unsupported encryption protocol '%s'."),
-				     protocol);
-			
-			return NULL;
-		}
-	} else if (supported != NULL) {
-		/* *shrug* - I guess just go on as if they match? */
-		protocol = supported;
-	} else {
+	if (!(protocol = g_mime_object_get_content_type_parameter ((GMimeObject *) encrypted, "protocol"))) {
 		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
 				     _("Cannot decrypt multipart/encrypted part: unspecified encryption protocol."));
 		
 		return NULL;
 	}
 	
-	version = g_mime_multipart_get_part (GMIME_MULTIPART (mpe), GMIME_MULTIPART_ENCRYPTED_VERSION);
-	
-	/* make sure the protocol matches the version part's content-type */
-	content_type = g_mime_content_type_to_string (version->content_type);
-	if (g_ascii_strcasecmp (content_type, protocol) != 0) {
-		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
-				     _("Cannot decrypt multipart/encrypted part: content-type does not match protocol."));
-		
-		g_free (content_type);
+	if (!(ctx = g_mime_crypto_context_new (protocol))) {
+		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
+			     _("Cannot decrypt multipart/encrypted part: unregistered encryption protocol '%s'."),
+			     protocol);
 		
 		return NULL;
 	}
-	g_free (content_type);
+	
+	supported = g_mime_crypto_context_get_encryption_protocol (ctx);
+	
+	/* make sure the protocol matches the crypto encrypt protocol */
+	if (!supported || g_ascii_strcasecmp (supported, protocol) != 0) {
+		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
+			     _("Cannot decrypt multipart/encrypted part: unsupported encryption protocol '%s'."),
+			     protocol);
+		g_object_unref (ctx);
+		
+		return NULL;
+	}
+	
+	version_part = g_mime_multipart_get_part ((GMimeMultipart *) encrypted, GMIME_MULTIPART_ENCRYPTED_VERSION);
+	
+	/* make sure the protocol matches the version part's content-type */
+	mime_type = g_mime_content_type_get_mime_type (version_part->content_type);
+	if (g_ascii_strcasecmp (mime_type, protocol) != 0) {
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
+				     _("Cannot decrypt multipart/encrypted part: content-type does not match protocol."));
+		
+		g_object_unref (ctx);
+		g_free (mime_type);
+		
+		return NULL;
+	}
+	g_free (mime_type);
 	
 	/* get the encrypted part and check that it is of type application/octet-stream */
-	encrypted = g_mime_multipart_get_part (GMIME_MULTIPART (mpe), GMIME_MULTIPART_ENCRYPTED_CONTENT);
-	mime_type = g_mime_object_get_content_type (encrypted);
-	if (!g_mime_content_type_is_type (mime_type, "application", "octet-stream")) {
+	encrypted_part = g_mime_multipart_get_part ((GMimeMultipart *) encrypted, GMIME_MULTIPART_ENCRYPTED_CONTENT);
+	content_type = g_mime_object_get_content_type (encrypted_part);
+	if (!g_mime_content_type_is_type (content_type, "application", "octet-stream")) {
 		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
 				     _("Cannot decrypt multipart/encrypted part: unexpected content type."));
+		g_object_unref (ctx);
 		
 		return NULL;
 	}
 	
 	/* get the ciphertext stream */
-	wrapper = g_mime_part_get_content_object (GMIME_PART (encrypted));
-	ciphertext = g_mime_data_wrapper_get_decoded_stream (wrapper);
+	content = g_mime_part_get_content ((GMimePart *) encrypted_part);
+	ciphertext = g_mime_stream_mem_new ();
+	g_mime_data_wrapper_write_to_stream (content, ciphertext);
 	g_mime_stream_reset (ciphertext);
 	
 	stream = g_mime_stream_mem_new ();
-	filtered_stream = g_mime_stream_filter_new (stream);
-	crlf_filter = g_mime_filter_crlf_new (FALSE, FALSE);
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), crlf_filter);
-	g_object_unref (crlf_filter);
+	filtered = g_mime_stream_filter_new (stream);
+	filter = g_mime_filter_dos2unix_new (FALSE);
+	g_mime_stream_filter_add ((GMimeStreamFilter *) filtered, filter);
+	g_object_unref (filter);
 	
 	/* get the cleartext */
-	if (!(res = g_mime_crypto_context_decrypt (ctx, ciphertext, filtered_stream, err))) {
-		g_object_unref (filtered_stream);
+	if (!(res = g_mime_crypto_context_decrypt (ctx, flags, session_key, ciphertext, filtered, err))) {
 		g_object_unref (ciphertext);
+		g_object_unref (filtered);
 		g_object_unref (stream);
+		g_object_unref (ctx);
 		
 		return NULL;
 	}
 	
-	g_mime_stream_flush (filtered_stream);
-	g_object_unref (filtered_stream);
+	g_mime_stream_flush (filtered);
 	g_object_unref (ciphertext);
+	g_object_unref (filtered);
+	g_object_unref (ctx);
 	
 	g_mime_stream_reset (stream);
 	parser = g_mime_parser_new ();
 	g_mime_parser_init_with_stream (parser, stream);
 	g_object_unref (stream);
 	
-	decrypted = g_mime_parser_construct_part (parser);
+	decrypted = g_mime_parser_construct_part (parser, NULL);
 	g_object_unref (parser);
 	
 	if (!decrypted) {
