@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -29,10 +29,11 @@
 #include "gmime-multipart-encrypted.h"
 #include "gmime-message-part.h"
 #include "gmime-stream-filter.h"
+#include "gmime-filter-unix2dos.h"
 #include "gmime-filter-strip.h"
 #include "gmime-filter-from.h"
-#include "gmime-filter-crlf.h"
 #include "gmime-stream-mem.h"
+#include "gmime-internal.h"
 #include "gmime-parser.h"
 #include "gmime-error.h"
 #include "gmime-part.h"
@@ -57,7 +58,6 @@
  * multipart/signed MIME type.
  **/
 
-
 /* GObject class methods */
 static void g_mime_multipart_signed_class_init (GMimeMultipartSignedClass *klass);
 static void g_mime_multipart_signed_init (GMimeMultipartSigned *mps, GMimeMultipartSignedClass *klass);
@@ -141,10 +141,10 @@ g_mime_multipart_signed_new (void)
 	GMimeMultipartSigned *multipart;
 	GMimeContentType *content_type;
 	
-	multipart = g_object_newv (GMIME_TYPE_MULTIPART_SIGNED, 0, NULL);
+	multipart = g_object_new (GMIME_TYPE_MULTIPART_SIGNED, NULL);
 	
 	content_type = g_mime_content_type_new ("multipart", "signed");
-	g_mime_object_set_content_type (GMIME_OBJECT (multipart), content_type);
+	g_mime_object_set_content_type ((GMimeObject *) multipart, content_type);
 	g_object_unref (content_type);
 	
 	return multipart;
@@ -185,59 +185,54 @@ sign_prepare (GMimeObject *mime_part)
 		subpart = GMIME_MESSAGE_PART (mime_part)->message->mime_part;
 		sign_prepare (subpart);
 	} else {
-		encoding = g_mime_part_get_content_encoding (GMIME_PART (mime_part));
+		encoding = g_mime_part_get_content_encoding ((GMimePart *) mime_part);
 		
 		if (encoding != GMIME_CONTENT_ENCODING_BASE64)
-			g_mime_part_set_content_encoding (GMIME_PART (mime_part),
-							  GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE);
+			g_mime_part_set_content_encoding ((GMimePart *) mime_part, GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE);
 	}
 }
 
 
 /**
  * g_mime_multipart_signed_sign:
- * @mps: multipart/signed object
- * @content: MIME part to sign
- * @ctx: encryption crypto context
+ * @ctx: a #GMimeCryptoContext
+ * @entity: MIME part to sign
  * @userid: user id to sign with
- * @digest: preferred digest algorithm
- * @err: exception
+ * @err: a #GError
  *
  * Attempts to sign the @content MIME part with @userid's private key
- * using the @ctx signing context with the @digest algorithm. If
- * successful, the signed #GMimeObject is set as the signed part of
- * the multipart/signed object @mps.
+ * using the @ctx signing context. If successful, a new multipart/signed
+ * object is returned.
  *
- * Returns: %0 on success or %-1 on fail. If the signing fails, an
- * exception will be set on @err to provide information as to why the
- * failure occured.
+ * Returns: (nullable) (transfer full): a new #GMimeMultipartSigned object on success
+ * or %NULL on fail. If signing fails, an exception will be set on @err to provide
+ * information as to why the failure occured.
  **/
-int
-g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
-			      GMimeCryptoContext *ctx, const char *userid,
-			      GMimeDigestAlgo digest, GError **err)
+GMimeMultipartSigned *
+g_mime_multipart_signed_sign (GMimeCryptoContext *ctx, GMimeObject *entity,
+			      const char *userid, GError **err)
 {
 	GMimeStream *stream, *filtered, *sigstream;
 	GMimeContentType *content_type;
-	GMimeDataWrapper *wrapper;
+	GMimeDataWrapper *content;
+	GMimeMultipartSigned *mps;
 	GMimePart *signature;
 	GMimeFilter *filter;
 	GMimeParser *parser;
 	const char *protocol;
 	const char *micalg;
-	int rv;
+	int algo;
 	
-	g_return_val_if_fail (GMIME_IS_MULTIPART_SIGNED (mps), -1);
-	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), -1);
-	g_return_val_if_fail (GMIME_IS_OBJECT (content), -1);
+	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
+	g_return_val_if_fail (GMIME_IS_OBJECT (entity), NULL);
 	
 	if (!(protocol = g_mime_crypto_context_get_signature_protocol (ctx))) {
 		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED, _("Signing not supported."));
-		return -1;
+		return NULL;
 	}
 	
 	/* Prepare all the parts for signing... */
-	sign_prepare (content);
+	sign_prepare (entity);
 	
 	/* get the cleartext */
 	stream = g_mime_stream_mem_new ();
@@ -245,63 +240,59 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	
 	/* Note: see rfc3156, section 3 - second note */
 	filter = g_mime_filter_from_new (GMIME_FILTER_FROM_MODE_ARMOR);
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered), filter);
+	g_mime_stream_filter_add ((GMimeStreamFilter *) filtered, filter);
 	g_object_unref (filter);
 	
 	/* Note: see rfc3156, section 5.4 (this is the main difference between rfc2015 and rfc3156) */
 	filter = g_mime_filter_strip_new ();
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered), filter);
+	g_mime_stream_filter_add ((GMimeStreamFilter *) filtered, filter);
 	g_object_unref (filter);
 	
-	g_mime_object_write_to_stream (content, filtered);
+	/* write the entity out to the stream */
+	g_mime_object_write_to_stream (entity, NULL, filtered);
 	g_mime_stream_flush (filtered);
-	g_object_unref (filtered);
 	g_mime_stream_reset (stream);
+	g_object_unref (filtered);
 	
-	/* Note: see rfc2015 or rfc3156, section 5.1 */
+	/* Note: see rfc2015 or rfc3156, section 5.1 - we do this *after* writing out
+	 * the entity because we'll end up parsing the mime part back out again and
+	 * we don't want it to be in DOS format. */
 	filtered = g_mime_stream_filter_new (stream);
-	filter = g_mime_filter_crlf_new (TRUE, FALSE);
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered), filter);
+	filter = g_mime_filter_unix2dos_new (FALSE);
+	g_mime_stream_filter_add ((GMimeStreamFilter *) filtered, filter);
 	g_object_unref (filter);
 	
 	/* construct the signature stream */
 	sigstream = g_mime_stream_mem_new ();
 	
 	/* sign the content stream */
-	if ((rv = g_mime_crypto_context_sign (ctx, userid, digest, filtered, sigstream, err)) == -1) {
+	if ((algo = g_mime_crypto_context_sign (ctx, TRUE, userid, filtered, sigstream, err)) == -1) {
 		g_object_unref (sigstream);
 		g_object_unref (filtered);
 		g_object_unref (stream);
-		return -1;
+		return NULL;
 	}
 	
 	g_object_unref (filtered);
 	g_mime_stream_reset (sigstream);
 	g_mime_stream_reset (stream);
 	
-	/* set the multipart/signed protocol and micalg */
-	content_type = g_mime_object_get_content_type (GMIME_OBJECT (mps));
-	g_mime_content_type_set_parameter (content_type, "protocol", protocol);
-	micalg = g_strdup (g_mime_crypto_context_digest_name (ctx, (GMimeDigestAlgo) rv));
-	g_mime_content_type_set_parameter (content_type, "micalg", micalg);
-	g_mime_multipart_set_boundary (GMIME_MULTIPART (mps), NULL);
-	
 	/* construct the content part */
 	parser = g_mime_parser_new_with_stream (stream);
-	content = g_mime_parser_construct_part (parser);
+	entity = g_mime_parser_construct_part (parser, NULL);
 	g_object_unref (stream);
 	g_object_unref (parser);
 	
 	/* construct the signature part */
-	content_type = g_mime_content_type_new_from_string (protocol);
+	content_type = g_mime_content_type_parse (NULL, protocol);
 	signature = g_mime_part_new_with_type (content_type->type, content_type->subtype);
 	g_object_unref (content_type);
 	
-	wrapper = g_mime_data_wrapper_new ();
-	g_mime_data_wrapper_set_stream (wrapper, sigstream);
-	g_mime_part_set_content_object (signature, wrapper);
+	content = g_mime_data_wrapper_new ();
+	g_mime_data_wrapper_set_stream (content, sigstream);
+	g_mime_part_set_content (signature, content);
 	g_object_unref (sigstream);
-	g_object_unref (wrapper);
+	g_object_unref (content);
 	
 	/* FIXME: temporary hack, this info should probably be set in
 	 * the CryptoContext class - maybe ::sign can take/output a
@@ -312,13 +303,19 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	}
 	
 	/* save the content and signature parts */
-	/* FIXME: make sure there aren't any other parts?? */
-	g_mime_multipart_add (GMIME_MULTIPART (mps), content);
-	g_mime_multipart_add (GMIME_MULTIPART (mps), (GMimeObject *) signature);
+	mps = g_mime_multipart_signed_new ();
+	g_mime_multipart_add ((GMimeMultipart *) mps, entity);
+	g_mime_multipart_add ((GMimeMultipart *) mps, (GMimeObject *) signature);
 	g_object_unref (signature);
-	g_object_unref (content);
+	g_object_unref (entity);
 	
-	return 0;
+	/* set the multipart/signed protocol and micalg */
+	micalg = g_mime_crypto_context_digest_name (ctx, (GMimeDigestAlgo) algo);
+	g_mime_object_set_content_type_parameter ((GMimeObject *) mps, "protocol", protocol);
+	g_mime_object_set_content_type_parameter ((GMimeObject *) mps, "micalg", micalg);
+	g_mime_multipart_set_boundary ((GMimeMultipart *) mps, NULL);
+	
+	return mps;
 }
 
 static gboolean
@@ -358,34 +355,33 @@ check_protocol_supported (const char *protocol, const char *supported)
 
 /**
  * g_mime_multipart_signed_verify:
- * @mps: multipart/signed object
- * @ctx: encryption crypto context
- * @err: exception
+ * @mps: a #GMimeMultipartSigned
+ * @flags: a #GMimeVerifyFlags
+ * @err: a #GError
  *
  * Attempts to verify the signed MIME part contained within the
- * multipart/signed object @mps using the @ctx crypto context.
+ * multipart/signed object @mps.
  *
- * Returns: (transfer full): a new #GMimeSignatureList object on
+ * Returns: (nullable) (transfer full): a new #GMimeSignatureList object on
  * success or %NULL on fail. If the verification fails, an exception
  * will be set on @err to provide information as to why the failure
  * occured.
  **/
 GMimeSignatureList *
-g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCryptoContext *ctx,
-				GError **err)
+g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeVerifyFlags flags, GError **err)
 {
-	const char *supported, *protocol, *micalg;
+	const char *supported, *protocol;
 	GMimeObject *content, *signature;
 	GMimeStream *stream, *sigstream;
 	GMimeSignatureList *signatures;
-	GMimeStream *filtered_stream;
+	GMimeFormatOptions *options;
 	GMimeDataWrapper *wrapper;
-	GMimeFilter *crlf_filter;
+	GMimeCryptoContext *ctx;
 	GMimeDigestAlgo digest;
-	char *content_type;
+	GMimeFilter *filter;
+	char *mime_type;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART_SIGNED (mps), NULL);
-	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
 	
 	if (g_mime_multipart_get_count ((GMimeMultipart *) mps) < 2) {
 		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
@@ -393,82 +389,76 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCryptoContext *c
 		return NULL;
 	}
 	
-	protocol = g_mime_object_get_content_type_parameter (GMIME_OBJECT (mps), "protocol");
-	micalg = g_mime_object_get_content_type_parameter (GMIME_OBJECT (mps), "micalg");
-	
-	supported = g_mime_crypto_context_get_signature_protocol (ctx);
-	
-	if (protocol) {
-		/* make sure the protocol matches the crypto sign protocol */
-		if (!check_protocol_supported (protocol, supported)) {
-			g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
-				     _("Cannot verify multipart/signed part: unsupported signature protocol '%s'."),
-				     protocol);
-			return NULL;
-		}
-	} else if (supported != NULL) {
-		/* *shrug* - I guess just go on as if they match? */
-		protocol = supported;
-	} else {
+	if (!(protocol = g_mime_object_get_content_type_parameter ((GMimeObject *) mps, "protocol"))) {
 		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
 				     _("Cannot verify multipart/signed part: unspecified signature protocol."));
 		
 		return NULL;
 	}
 	
-	signature = g_mime_multipart_get_part (GMIME_MULTIPART (mps), GMIME_MULTIPART_SIGNED_SIGNATURE);
-	
-	/* make sure the protocol matches the signature content-type */
-	content_type = g_mime_content_type_to_string (signature->content_type);
-	if (g_ascii_strcasecmp (content_type, protocol) != 0) {
-		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
-				     _("Cannot verify multipart/signed part: signature content-type does not match protocol."));
-		g_free (content_type);
+	if (!(ctx = g_mime_crypto_context_new (protocol))) {
+		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
+			     _("Cannot verify multipart/signed part: unregistered signature protocol '%s'."),
+			     protocol);
 		
 		return NULL;
 	}
-	g_free (content_type);
 	
-	content = g_mime_multipart_get_part (GMIME_MULTIPART (mps), GMIME_MULTIPART_SIGNED_CONTENT);
+	supported = g_mime_crypto_context_get_signature_protocol (ctx);
+	
+	/* make sure the protocol matches the crypto sign protocol */
+	if (!check_protocol_supported (protocol, supported)) {
+		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
+			     _("Cannot verify multipart/signed part: unsupported signature protocol '%s'."),
+			     protocol);
+		g_object_unref (ctx);
+		
+		return NULL;
+	}
+	
+	signature = g_mime_multipart_get_part ((GMimeMultipart *) mps, GMIME_MULTIPART_SIGNED_SIGNATURE);
+	
+	/* make sure the protocol matches the signature content-type */
+	mime_type = g_mime_content_type_get_mime_type (signature->content_type);
+	if (g_ascii_strcasecmp (mime_type, protocol) != 0) {
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
+				     _("Cannot verify multipart/signed part: signature content-type does not match protocol."));
+		g_object_unref (ctx);
+		g_free (mime_type);
+		
+		return NULL;
+	}
+	g_free (mime_type);
+	
+	content = g_mime_multipart_get_part ((GMimeMultipart *) mps, GMIME_MULTIPART_SIGNED_CONTENT);
 	
 	/* get the content stream */
 	stream = g_mime_stream_mem_new ();
-	filtered_stream = g_mime_stream_filter_new (stream);
 	
 	/* Note: see rfc2015 or rfc3156, section 5.1 */
-	crlf_filter = g_mime_filter_crlf_new (TRUE, FALSE);
-	g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), crlf_filter);
-	g_object_unref (crlf_filter);
+	options = _g_mime_format_options_clone (NULL, FALSE);
+	g_mime_format_options_set_newline_format (options, GMIME_NEWLINE_FORMAT_DOS);
 	
-	g_mime_object_write_to_stream (content, filtered_stream);
-	g_mime_stream_flush (filtered_stream);
-	g_object_unref (filtered_stream);
+	g_mime_object_write_to_stream (content, options, stream);
+	g_mime_format_options_free (options);
 	g_mime_stream_reset (stream);
 	
 	/* get the signature stream */
-	wrapper = g_mime_part_get_content_object (GMIME_PART (signature));
-	
-	/* FIXME: temporary hack for Balsa to support S/MIME,
-	 * ::verify() should probably take a mime part so it can
-	 * decode this itself if it needs to. */
-	if (!g_ascii_strcasecmp (protocol, "application/pkcs7-signature") ||
-	    !g_ascii_strcasecmp (protocol, "application/x-pkcs7-signature")) {
-		sigstream = g_mime_stream_mem_new ();
-		g_mime_data_wrapper_write_to_stream (wrapper, sigstream);
-	} else {
-		sigstream = g_mime_data_wrapper_get_stream (wrapper);
-	}
+	wrapper = g_mime_part_get_content ((GMimePart *) signature);
 	
+	sigstream = g_mime_stream_mem_new ();
+	g_mime_data_wrapper_write_to_stream (wrapper, sigstream);
 	g_mime_stream_reset (sigstream);
 	
 	/* verify the signature */
-	digest = g_mime_crypto_context_digest_id (ctx, micalg);
-	signatures = g_mime_crypto_context_verify (ctx, digest, stream, sigstream, err);
+	signatures = g_mime_crypto_context_verify (ctx, flags, stream, sigstream, NULL, err);
 	
 	d(printf ("attempted to verify:\n----- BEGIN SIGNED PART -----\n%.*s----- END SIGNED PART -----\n",
 		  (int) GMIME_STREAM_MEM (stream)->buffer->len, GMIME_STREAM_MEM (stream)->buffer->data));
 	
+	g_object_unref (sigstream);
 	g_object_unref (stream);
+	g_object_unref (ctx);
 	
 	return signatures;
 }
