@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -56,7 +56,7 @@ g_mime_parse_content_type (const char **in, char **type, char **subtype)
 	register const char *inptr;
 	const char *start = *in;
 	
-	decode_lwsp (&start);
+	skip_cfws (&start);
 	inptr = start;
 	
 	/* decode the type */
@@ -66,7 +66,7 @@ g_mime_parse_content_type (const char **in, char **type, char **subtype)
 	*type = g_strndup (start, (size_t) (inptr - start));
 	
 	start = inptr;
-	decode_lwsp (&start);
+	skip_cfws (&start);
 	
 	/* check for type/subtype delimeter */
 	if (*start++ != '/') {
@@ -76,7 +76,7 @@ g_mime_parse_content_type (const char **in, char **type, char **subtype)
 		return FALSE;
 	}
 	
-	decode_lwsp (&start);
+	skip_cfws (&start);
 	inptr = start;
 	
 	/* decode the subtype */
@@ -101,41 +101,178 @@ g_mime_parse_content_type (const char **in, char **type, char **subtype)
 
 
 /**
- * g_mime_decode_lwsp:
- * @in: address of input text string
+ * g_mime_skip_comment:
+ * @in: address of input string
  *
- * Skips past any LWSP or rfc822 comments in *@in and updates @in.
+ * Skips a comment.
+ *
+ * Returns: %TRUE on success or %FALSE otherwise.
  **/
-void
-g_mime_decode_lwsp (const char **in)
+gboolean
+g_mime_skip_comment (const char **in)
 {
-	const char *inptr = *in;
+	register const char *inptr = *in;
+	int depth = 1;
 	
-	while (*inptr && (*inptr == '(' || is_lwsp (*inptr))) {
-		while (*inptr && is_lwsp (*inptr))
+	/* skip over the '(' */
+	inptr++;
+	while (*inptr && depth) {
+		if (*inptr == '\\' && *(inptr + 1))
 			inptr++;
+		else if (*inptr == '(')
+			depth++;
+		else if (*inptr == ')')
+			depth--;
 		
-		/* skip over any comments */
-		if (*inptr == '(') {
-			int depth = 1;
-			
-			inptr++;
-			while (*inptr && depth) {
-				if (*inptr == '\\' && *(inptr + 1))
-					inptr++;
-				else if (*inptr == '(')
-					depth++;
-				else if (*inptr == ')')
-					depth--;
-				
-				inptr++;
-			}
-		}
+		inptr++;
 	}
 	
 	*in = inptr;
+	
+	return depth == 0;
 }
 
+
+/**
+ * g_mime_skip_lwsp:
+ * @in: address of input string
+ *
+ * Skips whitespace.
+ *
+ * Returns: %TRUE if any input was skipped or %FALSE otherwise.
+ **/
+gboolean
+g_mime_skip_lwsp (const char **in)
+{
+	register const char *inptr = *in;
+	const char *start = inptr;
+	
+	while (is_lwsp (*inptr))
+		inptr++;
+	
+	*in = inptr;
+	
+	return inptr > start;
+}
+
+
+/**
+ * g_mime_skip_cfws:
+ * @in: address of input string
+ *
+ * Skips comments and whitespace.
+ *
+ * Returns: %TRUE on success or %FALSE on error.
+ **/
+gboolean
+g_mime_skip_cfws (const char **in)
+{
+	const char *inptr = *in;
+	
+	skip_lwsp (&inptr);
+	
+	while (*inptr == '(') {
+		if (!skip_comment (&inptr))
+			return FALSE;
+		
+		skip_lwsp (&inptr);
+	}
+	
+	*in = inptr;
+	
+	return TRUE;
+}
+
+
+/**
+ * g_mime_skip_quoted:
+ * @in: address of input string
+ *
+ * Skips a quoted string.
+ *
+ * Returns: %TRUE on success or %FALSE on error.
+ **/
+gboolean
+g_mime_skip_quoted (const char **in)
+{
+	register const char *inptr = *in;
+	gboolean escaped = FALSE;
+	
+	/* skip over leading '"' */
+	inptr++;
+	
+	while (*inptr) {
+		if (*inptr == '\\') {
+			escaped = !escaped;
+		} else if (!escaped) {
+			if (*inptr == '"')
+				break;
+		} else {
+			escaped = FALSE;
+		}
+		
+		inptr++;
+	}
+	
+	if (*inptr == '\0') {
+		*in = inptr;
+		
+		return FALSE;
+	}
+	
+	/* skip over the closing '"' */
+	inptr++;
+	
+	*in = inptr;
+	
+	return TRUE;
+}
+
+
+/**
+ * g_mime_skip_atom:
+ * @in: address of input string
+ *
+ * Skips an atom.
+ *
+ * Returns: %TRUE if any input was skipped or %FALSE otherwise.
+ **/
+gboolean
+g_mime_skip_atom (const char **in)
+{
+	register const char *inptr = *in;
+	const char *start = inptr;
+	
+	while (is_atom (*inptr))
+		inptr++;
+	
+	*in = inptr;
+	
+	return inptr > start;
+}
+
+
+/**
+ * g_mime_skip_word:
+ * @in: address of input string
+ *
+ * Skips a word token.
+ *
+ * Returns: %TRUE on success or %FALSE otherwise.
+ **/
+gboolean
+g_mime_skip_word (const char **in)
+{
+	if (**in == '"')
+		return skip_quoted (in);
+	
+	if (is_atom (**in))
+		return skip_atom (in);
+	
+	return FALSE;
+}
+
+
 static const char *
 decode_quoted_string (const char **in)
 {
@@ -195,7 +332,7 @@ g_mime_decode_word (const char **in)
 {
 	const char *inptr = *in;
 	
-	decode_lwsp (&inptr);
+	skip_cfws (&inptr);
 	if (*inptr == '"') {
 		*in = inptr;
 		return decode_quoted_string (in);
@@ -216,10 +353,11 @@ decode_subliteral (const char **in, GString *domain)
 			g_string_append_c (domain, *inptr);
 			inptr++;
 			got = TRUE;
-		} else if (is_lwsp (*inptr))
-			decode_lwsp (&inptr);
-		else
+		} else if (is_lwsp (*inptr)) {
+			skip_cfws (&inptr);
+		} else {
 			break;
+		}
 	}
 	
 	*in = inptr;
@@ -232,7 +370,7 @@ decode_domain_literal (const char **in, GString *domain)
 {
 	const char *inptr = *in;
 	
-	decode_lwsp (&inptr);
+	skip_cfws (&inptr);
 	while (*inptr && *inptr != ']') {
 		if (decode_subliteral (&inptr, domain) && *inptr == '.') {
 			g_string_append_c (domain, *inptr);
@@ -268,7 +406,7 @@ g_mime_decode_domain (const char **in, GString *domain)
 	
 	inptr = *in;
 	while (inptr && *inptr) {
-		decode_lwsp (&inptr);
+		skip_cfws (&inptr);
 		if (*inptr == '[') {
 			/* domain literal */
 			g_string_append_c (domain, '[');
@@ -294,7 +432,7 @@ g_mime_decode_domain (const char **in, GString *domain)
 		}
 		
 		save = inptr;
-		decode_lwsp (&inptr);
+		skip_cfws (&inptr);
 		if (*inptr != '.') {
 			inptr = save;
 			break;
@@ -308,3 +446,97 @@ g_mime_decode_domain (const char **in, GString *domain)
 	
 	return domain->len > initial;
 }
+
+
+char *
+g_mime_decode_addrspec (const char **in)
+{
+	const char *word, *inptr;
+	GString *addrspec;
+	char *str;
+	
+	skip_cfws (in);
+	inptr = *in;
+	
+	if (!(word = decode_word (&inptr))) {
+		w(g_warning ("No local-part in addr-spec: %s", *in));
+		return NULL;
+	}
+	
+	addrspec = g_string_new ("");
+	g_string_append_len (addrspec, word, (size_t) (inptr - word));
+	
+	/* get the rest of the local-part */
+	skip_cfws (&inptr);
+	while (*inptr == '.') {
+		g_string_append_c (addrspec, *inptr++);
+		if ((word = decode_word (&inptr))) {
+			g_string_append_len (addrspec, word, (size_t) (inptr - word));
+			skip_cfws (&inptr);
+		} else {
+			w(g_warning ("Invalid local-part in addr-spec: %s", *in));
+			goto exception;
+		}
+	}
+	
+	/* we should be at the '@' now... */
+	if (*inptr++ != '@') {
+		w(g_warning ("Invalid addr-spec; missing '@': %s", *in));
+		goto exception;
+	}
+	
+	g_string_append_c (addrspec, '@');
+	if (!decode_domain (&inptr, addrspec)) {
+		w(g_warning ("No domain in addr-spec: %s", *in));
+		goto exception;
+	}
+	
+	str = addrspec->str;
+	g_string_free (addrspec, FALSE);
+	
+	*in = inptr;
+	
+	return str;
+	
+ exception:
+	
+	g_string_free (addrspec, TRUE);
+	
+	return NULL;
+}
+
+char *
+g_mime_decode_msgid (const char **in)
+{
+	const char *inptr = *in;
+	char *msgid = NULL;
+	
+	skip_cfws (&inptr);
+	if (*inptr != '<') {
+		w(g_warning ("Invalid msg-id; missing '<': %s", *in));
+	} else {
+		inptr++;
+	}
+	
+	skip_cfws (&inptr);
+	if ((msgid = decode_addrspec (&inptr))) {
+		skip_cfws (&inptr);
+		if (*inptr != '>') {
+			w(g_warning ("Invalid msg-id; missing '>': %s", *in));
+		} else {
+			inptr++;
+		}
+		
+		*in = inptr;
+	} else {
+		w(g_warning ("Invalid msg-id; missing addr-spec: %s", *in));
+		*in = inptr;
+		while (*inptr && *inptr != '>')
+			inptr++;
+		
+		msgid = g_strndup (*in, (size_t) (inptr - *in));
+		*in = inptr;
+	}
+	
+	return msgid;
+}
